#!/usr/bin/python

"""High-level interface to adns."""

import adns
from exceptions import Exception

class Error(Exception): pass

class QueryEngine:

    callback_submit = None
    callback_submit_reverse = None
    callback_submit_reverse_any = None
    
    def __init__(self, s=None):
        self._s = s or adns.init(adns.iflags.noautosys)
        self._queries = {}

    def synchronous(self, qname, rr, flags=0):
        return self._s.synchronous(qname, rr, flags)

    def submit(self, qname, rr, flags=0, callback=None, extra=None):
        callback = callback or self.callback_submit
        if not callback: raise Error, "callback required"
        q = self._s.submit(qname, rr, flags)
        self._queries[q] = qname, rr, flags, callback, extra

    def submit_reverse(self, qname, rr, flags=0, callback=None, extra=None):
        callback = callback or self.callback_submit_reverse
        if not callback: raise Error, "callback required"
        q = self._s.submit_reverse(qname, rr, flags)
        self._queries[q] = qname, rr, flags, callback, extra

    def submit_reverse_any(self, qname, rr, flags=0,
                           callback=None, extra=None):
        callback = callback or self.callback_submit_reverse_any
        if not callback: raise Error, "callback required"
        q = self._s.submit_reverse_any(qname, rr, flags)
        self._queries[q] = qname, rr, flags, callback, extra

    def cancel(self, query):
        query.cancel()
        try: del self._queries[query]
        except KeyError: pass
        
    def run(self, timeout=0):
        for q in self._s.completed(timeout):
            answer = q.check()
            qname, rr, flags, callback, extra = self._queries[q]
            del self._queries[q]
            apply(callback, (answer, qname, rr, flags, extra))

    def finished(self):
        return not len(self._queries)

    def finish(self):
        while not self.finished():
            self.run(1)

    def run_max(self, max):
        from time import time
        quittime = time()+max
        while not self.finished() and time()<=quittime:
            self.run(1)

    def globalsystemfailure(self):
        self._s.globalsystemfailure()
        self._queries.clear()

init = QueryEngine



#!/usr/bin/env python
import os, sys, string
import ADNS

class DNSBL:

    """A class for defining various DNS-based blacklists."""

    def __init__(self, name, zone, URL='', results=None):
        """Create a DNS blacklist name, based on the given zone.
        If presently, URL is a template that produces a link
        back to information for a given address. results
        should map returned addresses to list codes."""

        self.name = name
        self.zone = zone
        self.URL = URL
        self.results = {}
        if results:
            for result, name in results.items(): self.result(result, name)

    def result(self, result, name):
        """Add a possible result set."""
        self.results[result] = name

    def getURL(self, ip):
        """Return a URL to information on the list of ip on this
        blacklist."""
        return self.URL % ip
    

class DNSBLQueryEngine(ADNS.QueryEngine):

    def __init__(self, s=None, blacklists=None):
        ADNS.QueryEngine.__init__(self, s)
        self.blacklists = {}
        self.dnsbl_results = {}
        if blacklists:
            for l in blacklists: self.blacklist(l)
            
    def blacklist(self, dnsbl):
        """Add a DNSBL."""
        self.blacklists[dnsbl.name] = dnsbl
        
    def submit_dnsbl(self, qname):
        from adns import rr
        for l, d in self.blacklists.items():
            self.dnsbl_results[qname] = []
            self.submit_reverse_any(qname, d.zone, rr.A,
                                    callback=self.dnsbl_callback,
                                    extra=l)

    def dnsbl_callback(self, answer, qname, rr, flags, l):
        if not answer[0]:
            for addr in answer[3]:
                self.dnsbl_results[qname].append( (
                    self.blacklists[l].results.get(addr, "%s-%s"%(l,addr)),
                    self.blacklists[l].getURL(qname)) )

if __name__ == "__main__":
    blacklists = [
        DNSBL('ORDB', 'relays.ordb.org.',
              'http://ordb.org/lookup?addr=%s',
              {'127.0.0.2': 'ORDB'}),
        DNSBL('DEVNULL', 'dev.null.dk.',
              'http://fabel.dk/relay/test/index.epl?ip=%s&send=Check',
              { '127.0.0.2': 'DEVNULL' }),
        ]

    s = DNSBLQueryEngine(blacklists=blacklists)
    for i in sys.argv[1:]:
        s.submit_dnsbl(i)
    s.finish()
    listed = s.dnsbl_results
    for k, v in listed.items():
        hits = []
        for l, url in v: hits.append(l)
        if len(listed) > 1:
            print "%s: %s" % (k, string.join(hits))
        else:
            print string.join(hits)
            
